package textBookEgs_Chapter3_exact;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;

/**
 * Encapsulation of LineDrawView, taken from:
 * 
 * - Olsen Fig 2.5, p. 19 (called DrawView in this eg)
 * 
 * - Fig 3.18 p. 70
 * 
 * @author mb
 * 
 */
public class LineDrawView extends Canvas implements LineDrawListener,
		MouseMotionListener, MouseListener {

	private LineDrawModel myModel;
	private boolean isCreatingLine = false;
	private int startX;
	private int startY;
	private int activeLineIndex;

	public LineDrawView(LineDrawModel model) {
		this.myModel = model;
		// to listen for event dispatched from the model, install listener on
		// the model
		myModel.addListener(this);
		// to listen for mouse motion events, install listener as below
		this.addMouseMotionListener(this);
		// to listen for mouse events (which are different that *mouse motion*
		// events, install listener as below
		this.addMouseListener(this);
	}

	/*
	 * Paint when the AWT tells us to...
	 */
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		for (int i = 0; i < myModel.getNumberOfLines(); i++) {
			g2.drawLine(myModel.getEndPoint1(i).x, myModel.getEndPoint1(i).y,
					myModel.getEndPoint2(i).x, myModel.getEndPoint2(i).y);
		}
	}

	public Dimension getPreferredSize() {
		return new Dimension(400, 200);
	}

	@Override
	public void lineWillChange(int index, int newX1, int newY1, int newX2,
			int newY2) {
		Line line = myModel.getLine(index);
		if (line != null) {
			Rectangle oldRegion = new Rectangle(line.getStartPoint().x,
					line.getStartPoint().y, line.getEndPoint().x,
					line.getEndPoint().y);
			this.invalidate(); // should selectively damage a region at this
								// point, but it is not possible so just damage
								// the whole graphics object
		}
		Rectangle newRegion = new Rectangle(newX1, newY1, newX2, newY2);

		this.invalidate(); // should selectively damage a region at this
		// point, but it is not possible so just damage
		// the whole graphics object
		this.repaint();
	}

	@Override
	public void modelHasChanged() {
		this.invalidate();
		this.repaint();
	}

	// The following methods are specified in the MouseListener

	@Override
	public void mouseClicked(MouseEvent e) {
		// as an exercise, experiment with removing and adding mouse listener
		// from view - you will see that this method is invoked only when the
		// mouse listener is installed
		System.out.println("Click on point: " + e.getPoint());
	}

	@Override
	public void mousePressed(MouseEvent e) {
		System.out.println("DDD");
		isCreatingLine = true;
		startX = e.getPoint().x;
		startY = e.getPoint().y;
		activeLineIndex = myModel.addLine(startX, startY, startX, startY);
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		System.out.println("sadsa");

		isCreatingLine = false;
		myModel.moveLine(activeLineIndex, startX, startY, e.getPoint().x,
				e.getPoint().y);
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	@Override
	public void mouseDragged(MouseEvent e) {
	}

	// The following methods are specified in the MouseMotionListener

	@Override
	public void mouseMoved(MouseEvent e) {
		if (isCreatingLine) {
			myModel.moveLine(activeLineIndex, startX, startY, e.getPoint().x,
					e.getPoint().y);
		}
	}

}
